<?php 

require_once dirname(__FILE__) . DIRECTORY_SEPARATOR . 'drupento.inc';
define('IN_DRUPENTO_CLI', 1);
define('EXIT_CODE_SUCCESS', 1);
define('EXIT_CODE_FAILURE', 2);

function _drupento_trim_cli_arg($arg) {
	if ( $arg ) {
		return trim($arg, '\'\"');
	}
	else {
		return null;
	}
}

function _drupento_cli_error_handler( $errno, $errstr, $errfile = null, $errline = null, $errcontext = null ) {
	drupento_debug( "PHP Error: errno: {$errno} errstr: {$errstr} errfile: {$errfile} errline: {$errline} errcontext: {$errcontext}" );	
}

function init() {
	//error_reporting(0);
	//ini_set('display_errors', 'Off');		
	global $argv;
	
	$docroot = _drupento_trim_cli_arg($argv[1]);	

	if ( !$docroot || !is_dir($docroot) ) {
		throw new DrupentoException("Docroot {$docroot} does not exist");
	}

	chdir($docroot);
	require_once 'includes' . DIRECTORY_SEPARATOR . 'bootstrap.inc';
	
	drupal_bootstrap(constant('DRUPAL_BOOTSTRAP_FULL'));	
	
	if( php_sapi_name() !== 'cli' ) {
		throw new DrupentoException("Attempted to cross the auth bridge outside of CLI");	
	}
}

$old_error_handler = set_error_handler( '_drupento_cli_error_handler' );

try {

	init();
	global $user;

	$email = _drupento_trim_cli_arg($argv[2]);
	$password = _drupento_trim_cli_arg($argv[3]);

	drupento_debug( "About to authenticate from CLI {$email}" );
	user_external_login_register($email, 'drupento_auth');
	
	if( $user->uid ) {
		session_write_close();
		$key = db_result(db_query('SELECT sid from {sessions} WHERE uid = %d ORDER BY timestamp DESC LIMIT 1', $user->uid));
		echo $key;
	}
	else {
		drupento_debug( "EXTERNAL LOGIN FAILED" );
	}
	
	set_error_handler( $old_error_handler );
	exit(constant('EXIT_CODE_SUCCESS'));
}
catch( Exception $e ) {
	drupento_log_error( $e );
	drupento_debug( $e );
	set_error_handler( $old_error_handler );
	exit(constant('EXIT_CODE_FAILURE'));
}
?>
